/*****************************************************************************
 Copyright (c) 1999-2000, Intel Corporation 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of Intel Corporation nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

****************************************************************************/

#include "hamcore.h"
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/sched.h>


//globals
unsigned int*      dsp_irq;
struct wait_queue* delta_msr_wait;
unsigned int*      io_base;
unsigned int*      io_range;
void (* rs_receive)(void);
void (* rs_sched_event)(void);
int (* Debug_Printf)(const char*, ...);
void (* kschedule)(void);

int hamcore_init(void * data)
{

   struct hamcore_data_struct* driver = (struct hamcore_data_struct*)data;
   Debug_Printf = printk;
   kschedule = schedule;
   printk(KERN_DEBUG"hamcore_init\n");
   if(data == NULL)
   {
      printk(KERN_ALERT"null structure passed to hamcore_init\n");
      return -1;
   }
   if(driver->version != HAMCORE_VERSION)
   {
      printk(KERN_ALERT"hamcore_init: structure version != %d\n",
             HAMCORE_VERSION);
      return -1;
   }

   if(driver->dsp_irq)
   {
      dsp_irq = driver->dsp_irq;
//      printk(KERN_DEBUG"hamcore_init:dsp_irq %x,%x,%x\n",
//            &dsp_irq, dsp_irq, *dsp_irq);
   }else
   {
      printk(KERN_ALERT"dsp_irq is null\n");
      return -1;
   }
   if(driver->io_base)
   {
      io_base = driver->io_base;
//      printk(KERN_DEBUG"hamcore_init:io_base %x,%x,%x\n",
//             &io_base,io_base, *io_base);
   }else
   {
      printk(KERN_ALERT"hamcore_init:io_base is null\n");
      return -1;
   }
   if(driver->io_range)
   {
      io_range = driver->io_range;
//      printk(KERN_DEBUG"hamcore_init:io_range addr=0x%X, val=%u\n",
//             io_range,*io_range);
   }else
   {
      printk(KERN_ALERT"hamcore_init:io_range is null\n");
      return -1;
   }
   if(driver->rs_receive)
   {
      rs_receive = driver->rs_receive;
//      printk(KERN_DEBUG"hamcore_init:rs_receive addr=0x%X\n",
//             rs_receive);
   }else
   {
      printk(KERN_ALERT"hamcore_init:rs_receive is null\n");
      return -1;
   }

   if(driver->rs_sched_event)
   {
      rs_sched_event = driver->rs_sched_event;
//      printk(KERN_DEBUG"hamcore_init:rs_sched_event addr=0x%X\n",
//             (unsigned int)rs_sched_event);
   }else
   {
      printk(KERN_ALERT"hamcore_init:rs_sched_event is null\n");
      return -1;
   }

   printk("hamcore_init:done\n");
   return 0;
}

int init_module(void)
{
	printk("hamcore module loaded\n");
	return 0;
}
void cleanup_module(void)
{
	printk("hamcore module cleanup\n");
}

void hamdelay(short x)
{
//   printk(KERN_INFO"hamdelay %d\n",x);
   mdelay(x);
}


