/*****************************************************************************
 Copyright (c) 1999-2000, Intel Corporation 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of Intel Corporation nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

****************************************************************************/


#ifndef HAMCORE_H
#define HAMCORE_H
#define HAMCORE_VERSION 20010109 //yyyymmdd

#include"hamdefs.h"
#include <linux/proc_fs.h>

#define INTERNATIONAL_CODE 1 
#define RELEASE_BUILD 1
#define MDM_MFG_ID "Intel Corporation"
#define MDM_MODEL_NAME "Intel HaM Data Fax"
#define PCI_VENDOR_ID_AMBIENT  0x1813
#define PCI_DEVICE_ID_CLM      0x4000
#define CLM_DRIVER_NAME        "serial"
#define CLM_DEVICE_NAME        "ham"
#define CLM_MAJOR              240 //start of local-experimental use
#define CLM_MINOR_START        1
#define CLM_PORTS              1


#if INTERNATIONAL_CODE == 1
#define CODE_BASE "INT"
#else
#define CODE_BASE "FCC"
#endif

#if RELEASE_BUILD == 1
#define PRE_PREFIX "PRE-"
#else
#define PRE_PREFIX ""
#endif


#define VERSION_MSG "HaM "PRE_PREFIX"Release 333-5 "CODE_BASE" - "__DATE__

struct hamcore_data_struct
{
   unsigned int version;   //==HAMCORE_VERSION
   unsigned int* dsp_irq;
   unsigned int* io_base;
   unsigned int* io_range;
   void (* rs_receive)(void);
   void (* rs_sched_event)(void);
};

typedef enum _DEVSTATE {
   OPENED,
   OPENING,
   CLOSED,
   CLOSING,
   STOPPED,
   REMOVED
} DEVSTATE, *PDEVSTATE;


/* PCI Configuration Register Information */
struct pci_info_str
{
   unsigned char  bus;
   unsigned char  funct;

   unsigned short vendor;
   unsigned short device;
   unsigned char  revision;

   unsigned int   class;
   unsigned char  cache_line;
   unsigned char  latency_timer;
   unsigned char  header_type;
   unsigned char  bist;

   unsigned int   address[6];
   unsigned int   mask[6];

   unsigned short subsystem_vendor;
   unsigned short subsystem_device;

   unsigned char  irq_line;
   unsigned char  irq_pin;
   unsigned char  min_gnt;
   unsigned char  max_lat;
};

int get_pci_info(struct pci_info_str *pci_info, unsigned short vendor,
   unsigned short device);


#define CPINT_STAT   0x01    /* control processor interrupt pending */
#define INTEV_REG    0xf7    /* interrupt event register */

/*     Value for the variable modem_DTE_xon_char   */
#define XON_CHAR           0x11

/*     Value for the variable modem_DTE_xoff_char   */
#define XOFF_CHAR          0x13

/*     Values for the variable acu_state   */
#define S_IDLE                     0
#define S_PARSE                    1
#define S_ORIG                     2
#define S_ANSW                     3
#define S_ONLINE                   4
#define S_HANGUP                   5
#define S_TEST                     6
#define S_RETRAIN                  7
#define STATE_MAX                  8


/* fax1 state definitions */
#define IDLE                0
#define CONFIGURE_FTM       1
#define EXECUTE_FTM         2
#define COMPLETE_FTM        3
#define CONFIGURE_FRM       4
#define EXECUTE_FRM         5
#define COMPLETE_FRM        6
#define CONFIGURE_FTH       7
#define EXECUTE_FTH         8
#define COMPLETE_FTH        9
#define CONFIGURE_FRH       10
#define EXECUTE_FRH         11
#define COMPLETE_FRH        12

#define BPS1200_75      0
#define BPS75_1200      1
#define BPS300          2
#define BPS600          3
#define BPS1200         4
#define BPS2400         5
#define BPS4800         6
#define BPS7200         7
#define BPS9600         8
#define BPS12000        9
#define BPS14400        10
#define BPS16800        11
#define BPS19200        12
#define BPS21600        13
#define BPS24000        14
#define BPS26400        15
#define BPS28000        16
#define BPS28800        17
#define BPS29333        18
#define BPS30666        19
#define BPS31200        20
#define BPS32000        21
#define BPS33333        22
#define BPS33600        23
#define BPS34666        24
#define BPS36000        25
#define BPS37333        26
#define BPS38666        27
#define BPS38400        28
#define BPS40000        29
#define BPS41333        30
#define BPS42666        31
#define BPS44000        32
#define BPS45333        33
#define BPS46666        34
#define BPS48000        35
#define BPS49333        36
#define BPS50666        37
#define BPS52000        38
#define BPS53333        39
#define BPS54666        40
#define BPS56000        41
#define BPS57333        42
#define BPS57600        43
#define BPS115200       44



/* External data */
extern ubyte acu_loop;
extern ubyte acu_enabled;
extern char mdm_mfg_id[64];
extern char mdm_model_name[64];
extern ubyte at_z_command;
extern ubyte fax_class;
extern ubyte fax1_state;
extern ubyte acu_state;
extern ubyte slG, slQ;
extern ubyte AFE_type;
extern char gpio_lpohd;
extern ubyte dte_to_modem_xon_char;
extern ubyte dte_to_modem_xoff_char;
extern ubyte mt_v90_configured;
extern struct wait_queue *delta_msr_wait;
extern ubyte fatal_error_occurred;

/* External function prototypes */
extern void  clm_bg(void);
extern void  clm_initialize(void);
extern void  clm_terminate(void);
extern void  clm_rx_int(void);
extern void  clm_tx_int(void);
extern void  clm_external_int(void);
extern void  clm_configure(void);
extern int   rs_readfile(char*,char**,off_t,int,int);
extern int   rs_readproc(char*, char**, off_t, int, int*, void*);
extern int   rs_writefile(struct file*,const char*,unsigned long, void*);
extern void  init_eeprom_data(void);
extern void  wwh_init(void);
extern void  CurrentCountry_init(void);
extern void  master_init(void);
extern void  line_int();
extern long  ResetDspInternal();
extern void  dspdrv_clear_dsp_interrupt();
extern void  dspdrv_SetCramISRCallBack(void (*callback)());
extern void  dspdrv_CommRamISR(void);
extern word  Update_CurrentCountry(uword);
extern void  acu_from_clear();
extern uword acu_from_free();
extern void  ACU_Main(void);
extern void  acu_to_clear();
extern void  acu_external_int(void);
extern void  acu_tx_int(void);
extern void  acu_io_enable(void);
extern void  acu_io_disable(void);
extern void  dte_external_int(void);
extern void  dte_tx_int(void);
extern void  dte_to_clear();
extern void  dte_from_clear();
extern uword dte_from_free();

extern ubyte get_register(uword);
extern int (*Debug_Printf)(const char*, ...);

#endif HAMCORE_H
