/*****************************************************************************
 Copyright (c) 1999-2000, Intel Corporation 

 All rights reserved.

 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
 this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation 
 and/or other materials provided with the distribution.

 3. Neither the name of Intel Corporation nor the names of its contributors 
 may be used to endorse or promote products derived from this software 
 without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

****************************************************************************/


#include "hamcore.h"
#include <linux/pci.h>
#include <asm/system.h>


/***************************************************************************/
/* Function:  get_pci_info                                                 */
/*                                                                         */
/* Remarks:                                                                */
/*                                                                         */
/* Inputs:                                                                 */
/*                                                                         */
/* Outputs:                                                                */
/***************************************************************************/
int get_pci_info(struct pci_info_str *pci_info, unsigned short vendor,
   unsigned short device)
{
   unsigned int address[6] =
   {
      PCI_BASE_ADDRESS_0,
      PCI_BASE_ADDRESS_1,
      PCI_BASE_ADDRESS_2,
      PCI_BASE_ADDRESS_3,
      PCI_BASE_ADDRESS_4,
      PCI_BASE_ADDRESS_5
   };

   int   status;
   int   i;
   unsigned int class_rev;

   if (!pci_present())
   {
      return (1);
   }

   /* attempt to find vendor's device */
   status = pcibios_find_device(vendor, device, 0, &pci_info->bus,
      &pci_info->funct);

   if (!status)
   {
      pcibios_read_config_word(pci_info->bus, pci_info->funct,
         PCI_VENDOR_ID, &pci_info->vendor);
      pcibios_read_config_word(pci_info->bus, pci_info->funct,
         PCI_DEVICE_ID, &pci_info->device);
      pcibios_read_config_dword(pci_info->bus, pci_info->funct,
         PCI_CLASS_REVISION, &class_rev);
      pci_info->revision = class_rev & 0xff;
      pci_info->class = class_rev >> 16;

      pcibios_read_config_byte(pci_info->bus, pci_info->funct,
         PCI_CACHE_LINE_SIZE, &pci_info->cache_line);
      pcibios_read_config_byte(pci_info->bus, pci_info->funct,
         PCI_LATENCY_TIMER, &pci_info->latency_timer);
      pcibios_read_config_byte(pci_info->bus, pci_info->funct,
         PCI_HEADER_TYPE, &pci_info->header_type);
      pcibios_read_config_byte(pci_info->bus, pci_info->funct,
         PCI_BIST, &pci_info->bist);

      for (i = 0; i < 6; i++)
      {
         pcibios_read_config_dword(pci_info->bus, pci_info->funct,
            address[i], &pci_info->address[i]);
         cli();
         pcibios_write_config_dword(pci_info->bus, pci_info->funct,
            address[i], ~0);
         pcibios_read_config_dword(pci_info->bus, pci_info->funct,
            address[i], &pci_info->mask[i]);
         pcibios_write_config_dword(pci_info->bus, pci_info->funct,
            address[i], pci_info->address[i]);
         sti();
      }

      pcibios_read_config_word(pci_info->bus, pci_info->funct,
         PCI_SUBSYSTEM_VENDOR_ID, &pci_info->subsystem_vendor);
      pcibios_read_config_word(pci_info->bus, pci_info->funct,
         PCI_SUBSYSTEM_ID, &pci_info->subsystem_device);

      pcibios_read_config_byte(pci_info->bus, pci_info->funct,
         PCI_INTERRUPT_LINE, &pci_info->irq_line);
      pcibios_read_config_byte(pci_info->bus, pci_info->funct,
         PCI_INTERRUPT_PIN, &pci_info->irq_pin);
      pcibios_read_config_byte(pci_info->bus, pci_info->funct,
         PCI_MIN_GNT, &pci_info->min_gnt);
      pcibios_read_config_byte(pci_info->bus, pci_info->funct,
         PCI_MAX_LAT, &pci_info->max_lat);
   }

   return (status);
}
